<?php
/**
 * Handles the capture process.
 * @package App\Import & Export (capture)
 */
final class MH_App_Capture {

	/**
	 * Current instance.
	 */
	private $instance = array();

	/**
	 * Constructor.
	 */
	public function __construct( $context ) {
		if ( ! current_user_can( 'switch_themes' ) ) {
			return false;
		}

		if ( ! $this->instance = mh_app_cache_get( $context, 'mh_app_capture' ) ) {
			return false;
		}
	}

	/**
	 * Import data.
	 */
	public function capture($layout_id) {
		$this->prevent_failure();
			$data = mh_app_request_layout($layout_id);

		if ( ! empty($data) ){
            $timestamp = $this->get_timestamp();
            $content_data = json_decode( $data, true ); 
            $content_data = $this->validate( $content_data );
            $content = $this->apply_query( $content_data['data']) ;
            $layout_content = $content['post_content'];
            $subject = $layout_content;

            if ( isset( $content_data['images'] ) ) {
                $images = $this->maybe_paginate_images( (array) $content_data['images'], 'upload_images', $timestamp );
                $subject = $this->replace_images_urls( $images, $layout_content );
              
            }
		
		if ( $subject ){
			echo $subject;
		}	
        }
	}

	private function apply_query( $data) {
		$new = array();
		foreach ( $data as $id ) {
			$new = array_merge($new , $id) ;
		}
		return $new;
	}

	private function chunk_images( $images, $method, $id, $chunk = 0 ) {
		$images_per_chunk = 7;
		$chunks           = 1;

		$paginate_images = apply_filters( 'mh_app_paginate_images', true );

		if ( $paginate_images && count( $images ) > $images_per_chunk ) {
			$chunks       = ceil( count( $images ) / $images_per_chunk );
			$slice        = $images_per_chunk * $chunk;
			$images       = array_slice( $images, $slice, $images_per_chunk );
			$images       = $this->$method( $images );
			$filesystem   = $this->get_filesystem();
			$temp_file_id = sanitize_file_name( "images_{$id}" );
			$temp_file    = $this->temp_file( $temp_file_id, 'mh_app_capture' );
			$temp_images  = json_decode( $filesystem->get_contents( $temp_file ), true );

			if ( is_array( $temp_images ) ) {
				$images = array_merge( $temp_images, $images );
			}

			if ( $chunk + 1 < $chunks ) {
				$filesystem->put_contents( $temp_file, wp_json_encode( (array) $images ) );
			} else {
				$this->delete_temp_files( 'mh_app_capture', array( $temp_file_id => $temp_file ) );
			}
		} else {
			$images = $this->$method( $images );
		}

		return array(
			'ready'  => $chunk + 1 >= $chunks,
			'chunks' => $chunks,
			'images' => $images,
		);
	}

	private function maybe_paginate_images( $images, $method, $timestamp ) {
		mh_app_nonce_verified_previously();

		$page = isset( $_POST['page'] ) ? (int) $_POST['page'] : 1;
		$result = $this->chunk_images( $images, $method, $timestamp, max( $page - 1, 0 ) );

		if ( ! $result['ready'] ) {
			wp_send_json( array(
				'page'        => $page,
				'total_pages' => $result['chunks'],
				'timestamp'   => $timestamp,
			) );
		}

		return $result['images'];
	}

	private function upload_images( $images ) {
		$filesystem = $this->set_filesystem();

		foreach ( $images as $key => $image ) {
			$basename    = sanitize_file_name( wp_basename( $image['url'] ) );
			$attachments = get_posts( array(
				'posts_per_page' => -1,
				'post_type'      => 'attachment',
				'meta_key'       => '_wp_attached_file',
				'meta_value'     => pathinfo( $basename, PATHINFO_FILENAME ),
				'meta_compare'   => 'LIKE',
			) );
			$id = 0;
			$url = '';

			if ( ! is_wp_error( $attachments ) && ! empty( $attachments ) ) {
				foreach ( $attachments as $attachment ) {
					$attachment_url = wp_get_attachment_url( $attachment->ID );
					$file           = get_attached_file( $attachment->ID );
					$filename       = sanitize_file_name( wp_basename( $file ) );

					if ( $filesystem->get_contents( $file ) === base64_decode( $image['encoded'] ) ) {
						$id = isset( $image['id'] ) ? $attachment->ID : 0;
						$url = $attachment_url;

						break;
					}
				}
			}
      
			if ( empty( $url ) ) {
				$temp_file = wp_tempnam();
				$filesystem->put_contents( $temp_file, base64_decode( $image['encoded'] ) );
				$filetype = wp_check_filetype_and_ext( $temp_file, $basename );
        
				if ( isset( $filetype['proper_filename'] ) && $filetype['proper_filename'] !== $basename ) {
					if ( isset( $filename ) && $filename === $filetype['proper_filename'] ) {
						if ( $filesystem->get_contents( $file ) === $filesystem->get_contents( $temp_file ) ) {
							$filesystem->delete( $temp_file );
							continue;
						}
					}
				}

				$file = array(
					'name'     => $basename,
					'tmp_name' => $temp_file,
				);
				$upload = media_handle_sideload( $file, 0 );

				if ( ! is_wp_error( $upload ) ) {
					$id = isset( $image['id'] ) ? $upload : 0;
					$url = wp_get_attachment_url( $upload );
				} else {
					$filesystem->delete( $temp_file );
				}
			}

			if ( $id > 0 ) {
				$images[$key]['replacement_id'] = $id;
			}

			if ( ! empty( $url ) ) {
				$images[$key]['replacement_url'] = $url;
			}

			unset( $url );
		}

		return $images;
	}

	private function replace_image_url( $subject, $image ) {
		if ( isset( $image['replacement_id'] ) && isset( $image['id'] ) ) {
			$search      = $image['id'];
			$replacement = $image['replacement_id'];
			$subject     = preg_replace( "/(gallery_ids=.*){$search}(.*\")/", "\${1}{$replacement}\${2}", $subject );
		}

		if ( isset( $image['url'] ) && isset( $image['replacement_url'] ) && $image['url'] !== $image['replacement_url'] ) {
			$search      = $image['url'];
			$replacement = $image['replacement_url'];
			$subject     = str_replace( $search, $replacement, $subject );
		}

		return $subject;
	}

	private function replace_images_urls( $images, $data ) {
  				$data = wp_json_encode( $data );

		foreach ( $images as $image ) {
			if ( isset( $image['replacement_url'] ) ) {
				if ( isset( $image['id'] ) && is_int( $image['replacement_url'] ) ) {
					$search = $image['id'];
					$replacement = $image['replacement_url'];
					$data = preg_replace( "/(gallery_ids=.*){$search}(.*\")/", "\${1}{$replacement}\${2}", $data );
				} else {
					$url = str_replace( '/', '\/', $image['url'] );
					$replacement = str_replace( '/', '\/', $image['replacement_url'] );
					$data = str_replace( $url, $replacement, $data );
				}
			}
		}

		return json_decode( $data, true );
  }

	private function validate( $data, $fields_validation = array() ) {
		if ( ! is_array( $data ) ) {
			return false;
		}

		foreach ( $data as $key => $value ) {
			if ( is_array( $value ) ) {
				$data[$key] = $this->validate( $value, $fields_validation );
			} else {
				if ( isset( $fields_validation[$key] ) ) {
					$data[$key] = call_user_func( $fields_validation[$key], $value );
				} else {
					$data[$key] = wp_kses_post( $value );
				}
			}
		}

		return $data;
	}

	private function prevent_failure() {
		@set_time_limit( 0 );

		if ( (int) @ini_get( 'memory_limit' ) < 256 ) {
			@ini_set( 'memory_limit', '256M' );
		}
	}

	private function set_filesystem() {
		global $wp_filesystem;

		add_filter( 'filesystem_method', array( $this, 'replace_filesystem_method' ) );
		WP_Filesystem();

		return $wp_filesystem;
	}
	
	private function get_filesystem() {
		static $filesystem = null;

		if ( null === $filesystem ) {
			$filesystem = $this->set_filesystem();
		}

		return $filesystem;
	}
	
	private function has_temp_file( $id, $group ) {
		$temp_files = get_option( '_mh_app_capture_temp_files', array() );

		if ( isset( $temp_files[$group][$id] ) && file_exists( $temp_files[$group][$id] ) ) {
			return $temp_files[$group][$id];
		}

		return false;
	}

	private function temp_file( $id, $group, $temp_file = false, $content = '' ) {
		$temp_files = get_option( '_mh_app_capture_temp_files', array() );

		if ( ! isset( $temp_files[$group] ) ) {
			$temp_files[$group] = array();
		}

		if ( isset( $temp_files[$group][$id] ) && file_exists( $temp_files[$group][$id] ) ) {
			return $temp_files[$group][$id];
		}

		$temp_file = $temp_file ? $temp_file : wp_tempnam();
		$temp_files[$group][$id] = $temp_file;

		update_option( '_mh_app_capture_temp_files', $temp_files, false );

		if ( ! empty( $content ) ) {
			$this->get_filesystem()->put_contents( $temp_file, $content );
		}

		return $temp_file;
	}

	public function delete_temp_files( $group = false, $defined_files = false ) {
		$filesystem = $this->set_filesystem();
		$temp_files = get_option( '_mh_app_capture_temp_files', array() );

		if ( $group === true ) {
			foreach( $temp_files as $group_id => $_group ) {
				$this->delete_temp_files( $group_id );
			}
		}

		if ( ! isset( $temp_files[$group] ) ) {
			return;
		}

		$delete_files = ( is_array( $defined_files ) && ! empty( $defined_files ) ) ? $defined_files : $temp_files[$group];

		foreach ( $delete_files as $id => $temp_file ) {
			if ( isset( $temp_files[$group][$id] ) && $filesystem->delete( $temp_files[$group][$id] ) ) {
				unset( $temp_files[$group][$id] );
			}
		}

		if ( empty( $temp_files[$group] ) ) {
			unset( $temp_files[$group] );
		}

		if ( empty( $temp_files ) ) {
			delete_option( '_mh_app_capture_temp_files' );
		} else {
			update_option( '_mh_app_capture_temp_files', $temp_files, false );
		}
	}

	public function replace_filesystem_method() {
		return 'direct';
	}

	public function get_timestamp() {
		return isset( $_POST['timestamp'] ) && ! empty( $_POST['timestamp'] ) ? sanitize_text_field( $_POST['timestamp'] ) : current_time( 'timestamp' );
	}
}