<?php
/**
 * Gets option value from the single theme option, stored as an array in the database
 * if all options stored in one row.
 * Stores the serialized array with theme options into the global variable on the first function run on the page.
 *
 * If options are stored as separate rows in database, it simply uses get_option() function.
 *
 * @param string $option_name Theme option name.
 * @param string $default_value Default value that should be set if the theme option isn't set.
 * @param string $used_for_object "Object" name that should be translated into corresponding "object" if WPML is activated.
 * @return mixed Theme option value or false if not found.
 */
if ( ! function_exists( 'mh_get_option' ) ) :
function mh_get_option( $option_name, $default_value = '', $used_for_object = '', $force_default_value = false ) {
	global $mh_composer_plugin_options;

	$shortname = 'page_composer_plugin';

	$mh_theme_options_name = 'mh_' . $shortname;

	if ( ! isset( $mh_composer_plugin_options ) ) {
		$mh_composer_plugin_options = get_option( $mh_theme_options_name );
	}
	$option_value = isset ( $mh_composer_plugin_options[$option_name] ) ? $mh_composer_plugin_options[$option_name] : false;

	// option value might be equal to false, so check if the option is not set in the database
	if ( ! isset( $mh_composer_plugin_options[ $option_name ] ) && ( '' != $default_value || $force_default_value ) ) {
		$option_value = $default_value;
	}

	if ( '' != $used_for_object && in_array( $used_for_object, array( 'page', 'category' ) ) && is_array( $option_value ) ) {
		$option_value = mh_generate_wpml_ids( $option_value, $used_for_object );
	}

	return $option_value;
}
endif;

if ( ! function_exists( 'mh_update_option' ) ) :
function mh_update_option( $option_name, $new_value ){
	global $mh_composer_plugin_options;

	$shortname = 'page_composer_plugin';

	$mh_theme_options_name = 'mh_' . $shortname;

	if ( ! isset( $mh_composer_plugin_options ) ) $mh_composer_plugin_options = get_option( $mh_theme_options_name );
	$mh_composer_plugin_options[$option_name] = $new_value;

	$option_name = $mh_theme_options_name;
	$new_value = $mh_composer_plugin_options;

	update_option( $option_name, $new_value );
}
endif;

if ( ! function_exists( 'mh_delete_option' ) ) :
function mh_delete_option( $option_name ){
	global $mh_composer_plugin_options;

	$shortname = 'page_composer_plugin';

	$mh_theme_options_name = 'mh_' . $shortname;

	if ( ! isset( $mh_composer_plugin_options ) ) $mh_composer_plugin_options = get_option( $mh_theme_options_name );

	unset( $mh_composer_plugin_options[$option_name] );
	update_option( $mh_theme_options_name, $mh_composer_plugin_options );
}
endif;

/* this function gets thumbnail from Post Thumbnail or Custom field or First post image */
if ( ! function_exists( 'get_thumbnail' ) ) :
function get_thumbnail($width=100, $height=100, $class='', $alttext='', $titletext='', $fullpath=false, $custom_field='', $post='') {
	if ( $post == '' ) global $post;
	global $shortname;

	$thumb_array['thumb'] = '';
	$thumb_array['use_timthumb'] = true;
	if ($fullpath) $thumb_array['fullpath'] = ''; //full image url for lightbox

	$new_method = true;

	if ( has_post_thumbnail( $post->ID ) ) {
		$thumb_array['use_timthumb'] = false;

		$mh_fullpath = wp_get_attachment_image_src( get_post_thumbnail_id( $post->ID ), 'full' );
		$thumb_array['fullpath'] = $mh_fullpath[0];
		$thumb_array['thumb'] = $thumb_array['fullpath'];
	}

	if ($thumb_array['thumb'] == '') {
		if ($custom_field == '') $thumb_array['thumb'] = esc_attr( get_post_meta($post->ID, 'Thumbnail', $single = true) );
		else {
			$thumb_array['thumb'] = esc_attr( get_post_meta($post->ID, $custom_field, $single = true) );
			if ($thumb_array['thumb'] == '') $thumb_array['thumb'] = esc_attr( get_post_meta($post->ID, 'Thumbnail', $single = true) );
		}

		#if custom field used for small pre-cropped image, open Thumbnail custom field image in lightbox
		if ($fullpath) {
			$thumb_array['fullpath'] = $thumb_array['thumb'];
			if ($custom_field == '') $thumb_array['fullpath'] = apply_filters('mh_fullpath', mh_path_reltoabs(esc_attr($thumb_array['thumb'])));
			elseif ( $custom_field <> '' && get_post_meta($post->ID, 'Thumbnail', $single = true) ) $thumb_array['fullpath'] = apply_filters( 'mh_fullpath', mh_path_reltoabs(esc_attr(get_post_meta($post->ID, 'Thumbnail', $single = true))) );
		}
	}

	return $thumb_array;
}
endif;

/* this function prints thumbnail from Post Thumbnail or Custom field or First post image */
if ( ! function_exists( 'print_thumbnail' ) ) :
function print_thumbnail($thumbnail = '', $use_timthumb = true, $alttext = '', $width = 100, $height = 100, $class = '', $echoout = true, $forstyle = false, $resize = true, $post='', $mh_post_id = '' ) {
	if ( is_array( $thumbnail ) ){
		extract( $thumbnail );
	}

	if ( $post == '' ) global $post, $mh_theme_image_sizes;

	$output = '';

	$mh_post_id = '' != $mh_post_id ? (int) $mh_post_id : $post->ID;

	if ( has_post_thumbnail( $mh_post_id ) ) {
		$thumb_array['use_timthumb'] = false;

		$image_size_name = $width . 'x' . $height;
		$mh_size = isset( $mh_theme_image_sizes ) && array_key_exists( $image_size_name, $mh_theme_image_sizes ) ? $mh_theme_image_sizes[$image_size_name] : array( $width, $height );

		$mh_attachment_image_attributes = wp_get_attachment_image_src( get_post_thumbnail_id( $mh_post_id ), $mh_size );
		$thumbnail = $mh_attachment_image_attributes[0];
	}

	if ( false === $forstyle ) {
		$output = '<img src="' . esc_url( $thumbnail ) . '"';

		if ($class <> '') $output .= " class='" . esc_attr( $class ) . "' ";

		$dimensions = apply_filters( 'mh_print_thumbnail_dimensions', " width='" . esc_attr( $width ) . "' height='" .esc_attr( $height ) . "'" );

		$output .= " alt='" . esc_attr( strip_tags( $alttext ) ) . "'{$dimensions} />";

		if ( ! $resize ) $output = $thumbnail;
	} else {
		$output = $thumbnail;
	}

	if ($echoout) echo $output;
	else return $output;
}
endif;

if ( ! function_exists( 'mh_path_reltoabs' ) ) :
function mh_path_reltoabs( $imageurl ){
	if ( strpos(strtolower($imageurl), 'http://') !== false || strpos(strtolower($imageurl), 'https://') !== false ) return $imageurl;

	if ( strpos( strtolower($imageurl), $_SERVER['HTTP_HOST'] ) !== false )
		return $imageurl;
	else {
		$imageurl = esc_url( apply_filters( 'mh_path_relative_image', site_url() . '/' ) . $imageurl );
	}

	return $imageurl;
}
endif;

/*this function allows for the auto-creation of post excerpts*/
if ( ! function_exists( 'truncate_post' ) ) {

	function truncate_post( $amount, $echo = true, $post = '', $strip_shortcodes = false ) {
		global $shortname;

		if ( '' == $post ) global $post;

		$post_excerpt = '';
		$post_excerpt = apply_filters( 'the_excerpt', $post->post_excerpt );

		if ( true == get_theme_mod( $shortname . '_use_excerpt' ) && '' != $post_excerpt ) {
			if ( $echo ) echo $post_excerpt;
			else return $post_excerpt;
		} else {
			// get the post content
			$truncate = $post->post_content;

			// remove caption shortcode from the post content
			$truncate = preg_replace( '@\[caption[^\]]*?\].*?\[\/caption]@si', '', $truncate );

			// remove audio shortcode from post content to prevent unwanted audio file on the excerpt
			// due to unparsed audio shortcode
			$truncate = preg_replace( '@\[audio[^\]]*?\].*?\[\/audio]@si', '', $truncate );

			// remove embed shortcode from post content
			$truncate = preg_replace( '@\[embed[^\]]*?\].*?\[\/embed]@si', '', $truncate );
			
			// remove breadcrumbs shortcode from the post content
			$truncate = preg_replace( '@\[mh_breadcrumbs[^\]]*?\].*?\[\/mh_breadcrumbs]@si', '', $truncate );

			if ( $strip_shortcodes ) {
				$truncate = mh_strip_shortcodes( $truncate );
			} else {
				// apply content filters
				$truncate = apply_filters( 'the_content', $truncate );
			}

			// decide if we need to append dots at the end of the string
			if ( strlen( $truncate ) <= $amount ) {
				$echo_out = '';
			} else {
				$echo_out = '...';
				// $amount = $amount - 3;
			}

			// trim text to a certain number of characters, also remove spaces from the end of a string ( space counts as a character )
			$truncate = rtrim( mh_wp_trim_words( $truncate, $amount, '' ) );

			// remove the last word to make sure we display all words correctly
			if ( '' != $echo_out ) {
				$new_words_array = (array) explode( ' ', $truncate );
				array_pop( $new_words_array );

				$truncate = implode( ' ', $new_words_array );

				// append dots to the end of the string
				$truncate .= $echo_out;
			}

			if ( $echo ) echo $truncate;
			else return $truncate;
		};
	}

}
if ( ! function_exists( 'mh_wp_trim_words' ) ){
	function mh_wp_trim_words( $text, $num_words = 55, $more = null ) {
		if ( null === $more )
		$more = esc_html__('&hellip;');
		$original_text = $text;
		$text = wp_strip_all_tags( $text );

		$text = trim( preg_replace( "/[\n\r\t ]+/", ' ', $text ), ' ' );
		preg_match_all( '/./u', $text, $words_array );
		$words_array = array_slice( $words_array[0], 0, $num_words + 1 );
		$sep = '';

		if ( count( $words_array ) > $num_words ) {
			array_pop( $words_array );
			$text = implode( $sep, $words_array );
			$text = $text . $more;
		} else {
			$text = implode( $sep, $words_array );
		}

		//return apply_filters( 'wp_trim_words', $text, $num_words, $more, $original_text );
		return $text;
	}
}


/**
 * Fix to allow pagination in some components to work in non-hierarchical singular page.
 * WP_Query based components wouldn't work in non-hierarchical single post type page
 * due to canonical redirect to prevent page duplication which could lead to SEO penalty.
 * @todo Check applicable shortcodes
 */
function mhc_modify_canonical_redirect( $redirect_url, $requested_url ) {
    global $post;
    $allowed_shortcodes = array( 'mhc_blog', 'mhc_portfolio', 'mhc_magazine', 'mhc_fullwidth_magazine', 'mhc_ticker', 'mhc_fullwidth_ticker', 'mhc_fullwidth_portfolio', 'mhc_filterable_portfolio' );
    $is_overwrite_canonical_redirect = false;

    // Look for $allowed_shortcodes in content. Once detected, set $is_overwrite_canonical_redirect to true
    foreach ( $allowed_shortcodes as $shortcode ) {
        if ( !empty( $post ) && has_shortcode( $post->post_content, $shortcode ) ) {
            $is_overwrite_canonical_redirect = true;
            break;
        }
    }

    // Only alter canonical redirect if current page is singular, has paged and $allowed_shortcodes or current page is front_page, has page and $allowed_shortcodes
    if ( ( is_singular() & ! is_home() && get_query_var( 'paged' ) && $is_overwrite_canonical_redirect ) || ( is_front_page() && get_query_var( 'page' ) && $is_overwrite_canonical_redirect ) ) {
        return $requested_url;
    }

    return $redirect_url;
}
add_filter( 'redirect_canonical', 'mhc_modify_canonical_redirect', 10, 2 );

/**
 * Get an array of active required plugins and mharty plugins.
 * this is to check when installing plugins and applying layouts and sections etc.
 */
if ( ! function_exists( 'mharty_active_plugins' ) ) :
function mharty_active_plugins() {
    $plugins_array = array();
    
    if ( class_exists( 'MHComposer', false ) )
        $plugins_array[] = 'composer';
    if ( class_exists( 'MHShop', false ) )
        $plugins_array[] = 'shop';
	if ( class_exists( 'MHLoveitClass', false ) )
        $plugins_array[] = 'loveit';
    if ( class_exists( 'MHReviewsClass', false ) )
        $plugins_array[] = 'reviews';
    if( class_exists( 'MHMagazine', false ) )
        $plugins_array[] = 'magazine';
    if( class_exists( 'MHMoreIconsClass', false ) )
        $plugins_array[] = 'icons';
    if( class_exists( 'Mh_Author_Page_Cover', false ) )
        $plugins_array[] = 'author';
    if( is_plugin_active( 'mh-shortcodes/mh-shortcodes.php' ) )
        $plugins_array[] = 'shortcodes';
    if( is_plugin_active( 'classic-editor/classic-editor.php' ) )
        $plugins_array[] = 'classiceditor';
    if( is_plugin_active( 'classic-widgets/classic-widgets.php' ) )
        $plugins_array[] = 'classicwidgets';
    
    return $plugins_array;
}
endif;

if ( ! function_exists( 'mh_app_nonce_verified_previously' ) ) :
function mh_app_nonce_verified_previously() {
  //do nothing just a check.
}
endif;

/**
 * contact form 7 Activation hook
 */
function mh_composer_cf7_activate() {
    
  update_option( 'mh_composer_cf7_activated', true );

}
register_activation_hook( 'contact-form-7/wp-contact-form-7.php', 'mh_composer_cf7_activate' );



function mh_composer_cf7_load_plugin() {

    if ( is_admin() && get_option( 'mh_composer_cf7_activated' ) ) {

        delete_option( 'mh_composer_cf7_activated' );
          //Force clear MH Composer cache
        if ( function_exists( 'mh_update_option' ) ) {
          mh_update_option( 'mhc_clear_templates_cache', 'on' );
        }
    }
}
add_action( 'admin_init', 'mh_composer_cf7_load_plugin' );

function mh_composer_cf7_deactivate() {

  //Force clear MH Composer cache
  if ( function_exists( 'mh_update_option' ) ) {
    mh_update_option( 'mhc_clear_templates_cache', 'on' );
  }

}
register_deactivation_hook( 'contact-form-7/wp-contact-form-7.php', 'mh_composer_cf7_deactivate' );


/**
 * Is WooCommerce active?
 */
if ( ! function_exists( 'mh_is_woocommerce_active' ) ):
function mh_is_woocommerce_active() {
	return class_exists( 'WooCommerce' );
}
endif;

/**
 * bolck editor active?
 */
if ( ! function_exists( 'mh_is_gutenberg_active' ) ):
function mh_is_gutenberg_active() {
	global $wp_version;

	static $has_wp5_plus = null;

	if ( is_null( $has_wp5_plus ) ) {
		$has_wp5_plus = version_compare( $wp_version, '5.0-alpha1', '>=' );
	}

	return $has_wp5_plus || function_exists( 'is_gutenberg_page' );
}
endif;


/**
 * block editor active enabled for the current post
 */
if ( ! function_exists( 'mh_is_gutenberg_enabled' ) ):
function mh_is_gutenberg_enabled() {
	if ( function_exists( 'is_gutenberg_page' ) ) {
		return mh_is_gutenberg_active() && is_gutenberg_page() && has_filter( 'replace_editor', 'gutenberg_init' );
	}

	return mh_is_gutenberg_active() && function_exists( 'use_block_editor_for_post' ) && use_block_editor_for_post( null );
}
endif;

if ( ! function_exists( 'mh_passthru_unescaped' ) ) :
function mh_passthru_unescaped( $passthru, $reason ) {
	$all_reasons = array(
		'html',
		//'underscore_template',
        //'cap_based_sanitized',
		//'fixed_string',
	);

	if ( ! in_array( $reason, $all_reasons ) ) {
		_doing_it_wrong( __FUNCTION__, 'This is not a valid reason to not escape this value.', esc_html( mh_get_theme_version() ) );
	}

	return $passthru;
}
endif;

if ( ! function_exists( 'mh_array_get' ) ) :
function mh_array_get( $array, $address, $default = '' ) {
		$keys   = is_array( $address ) ? $address : explode( '.', $address );
		$value  = $array;

		foreach ( $keys as $key ) {
			if ( ! empty( $key ) && isset( $key[0] ) && '[' === $key[0] ) {
				$index = substr( $key, 1, -1 );

				if ( is_numeric( $index ) ) {
					$key = (int) $index;
				}
			}

			if ( ! isset( $value[ $key ] ) ) {
				return $default;
			}

			$value = $value[ $key ];
		}

		return $value;
}
endif;

if ( ! function_exists( 'mh_use_responsive_images' ) ):
function mh_use_responsive_images() {

    $enable_responsive_images = mh_get_option( 'mharty_disable_responsive_images', false);
    if ( null === $enable_responsive_images ) {
      $enable_responsive_images = 'on';
    }
  
	return $enable_responsive_images;
}
endif;

if ( ! function_exists( 'mh_device_breakpoints' ) ) :
function mh_device_breakpoints() {
	return array(
		'desktop' => 1280,
		'tablet'  => 980,
		'phone'   => 480,
	);
}
endif;

if ( ! function_exists( 'mh_get_responsive_images' ) ) :
function mh_get_responsive_images( $orig_width, $orig_height, $breakpoint ) {
	$mh_device_breakpoints = mh_device_breakpoints();

	if ( ! isset( $mh_device_breakpoints[ $breakpoint ] ) ) {
		return false;
	}

	$new_width = $mh_device_breakpoints[ $breakpoint ];

	if ( $new_width >= $orig_width ) {
		return false;
	}

	$ratio = ( $orig_width * 1.0 ) / $orig_height;

	$new_height = round( ( $new_width / $ratio ) );

	return array(
		'width'  => $new_width,
		'height' => $new_height,
	);
}
endif;

// Add srcset and sizes to img
if ( ! function_exists( 'mh_images_add_srcset_and_sizes' ) ) :
function mh_images_add_srcset_and_sizes( $image, $echo = false ) {
	static $srcset_and_sizes_cached = array();

	if (!mh_use_responsive_images() ) {
		if ( $echo ) {
			echo mh_passthru_unescaped( $image, 'html' );
		}
		return $image;
	}

	$src = mh_get_src_from_img_tag( $image );

	$cache_key = $src ? $src : 'empty-src';

	if ( isset( $srcset_and_sizes_cached[ $cache_key ] ) ) {
		$image = $srcset_and_sizes_cached[ $cache_key ];
	} else {

		if ( $src ) {
			$attachment_id = mh_get_attachment_id_by_url( $src );
			$image_meta    = false;

			if ( $attachment_id ) {
				$image_meta = wp_get_attachment_metadata( $attachment_id );
			}

			if ( $image_meta ) {
				$image = wp_image_add_srcset_and_sizes( $image, $image_meta, $attachment_id );
			}
		}

		$srcset_and_sizes_cached[ $cache_key ] = $image;
	}

	if ( $echo ) {
		echo mh_passthru_unescaped( $image, 'html' );
	}

	return $image;
}
endif;

if ( ! function_exists( 'mh_get_attachment_id_by_url_sql' ) ) :
	function mh_get_attachment_id_by_url_sql( $normalized_url ) {
		global $wpdb;
    
		$cleaned_url = preg_replace( '/^https?:/i', '', $normalized_url );
		$fallback_url = preg_replace( '/-(\d+)x(\d+)\.(jpg|jpeg|gif|png|svg|webp)$/', '.$3', $cleaned_url );

		if ( $cleaned_url === $fallback_url ) {
			$attachments_query = $wpdb->prepare(
				"SELECT id
				FROM $wpdb->posts
				WHERE `post_type` = %s
					AND `guid` IN ( %s, %s )",
				'attachment',
				esc_url_raw( "https:{$cleaned_url}" ),
				esc_url_raw( "http:{$cleaned_url}" )
			);
		} else {
			$attachments_query = $wpdb->prepare(
				"SELECT id
				FROM $wpdb->posts
				WHERE `post_type` = %s
					AND `guid` IN ( %s, %s, %s, %s )
				ORDER BY CHAR_LENGTH( `guid` ) DESC",
				'attachment',
				esc_url_raw( "https:{$cleaned_url}" ),
				esc_url_raw( "https:{$fallback_url}" ),
				esc_url_raw( "http:{$cleaned_url}" ),
				esc_url_raw( "http:{$fallback_url}" )
			);
		}

		return $attachments_query;
	}
endif;

if ( ! function_exists( 'mh_get_attachment_id_by_url' ) ) :
function mh_get_attachment_id_by_url( $url ) {
	global $wpdb;
	$attachment_id_pre = apply_filters( 'mh_get_attachment_id_by_url_pre', false, $url );

	if ( false !== $attachment_id_pre ) {
		return $attachment_id_pre;
	}

	$url = apply_filters( 'mh_get_attachment_id_by_url_guid', $url );
	$normalized_url = mh_attachment_normalize_url( $url );
  
	if ( ! $normalized_url ) {
		return 0;
	}

	$attachments_sql_query = mh_get_attachment_id_by_url_sql( $normalized_url );
	$attachment_id         = (int) $wpdb->get_var( $attachments_sql_query );

	if ( ! $attachment_id && false !== strpos( $normalized_url, '-scaled.' ) ) {
		$normalized_url_not_scaled = str_replace( '-scaled.', '.', $normalized_url );
		$attachments_sql_query     = mh_get_attachment_id_by_url_sql( $normalized_url_not_scaled );
		$attachment_id             = (int) $wpdb->get_var( $attachments_sql_query );
	}

	if ( ! $attachment_id ) {
		$uploads         = wp_get_upload_dir();
		$uploads_baseurl = trailingslashit( $uploads['baseurl'] );

		if ( 0 === strpos( $normalized_url, $uploads_baseurl ) ) {
			$file_path = str_replace( $uploads_baseurl, '', $normalized_url );
			$file_path_no_resize = preg_replace( '/-(\d+)x(\d+)\.(jpg|jpeg|gif|png|svg|webp)$/', '.$3', $file_path );

			if ( $file_path === $file_path_no_resize ) {
				$attachments_sql_query = $wpdb->prepare(
					"SELECT post_id
					FROM $wpdb->postmeta
					WHERE `meta_key` = %s
						AND `meta_value` = %s",
					'_wp_attached_file',
					$file_path
				);
			} else {
				$attachments_sql_query = $wpdb->prepare(
					"SELECT post_id
					FROM $wpdb->postmeta
					WHERE `meta_key` = %s
						AND `meta_value` IN ( %s, %s )
					ORDER BY CHAR_LENGTH( `meta_value` ) DESC",
					'_wp_attached_file',
					$file_path,
					$file_path_no_resize
				);
			}

			$attachment_id = (int) $wpdb->get_var( $attachments_sql_query );
		}
	}
  
	return $attachment_id;
}
endif;

if ( ! function_exists( 'mh_get_attachment_size_by_url' ) ) :
function mh_get_attachment_size_by_url( $url, $default_size = 'full' ) {
	$normalized_url = mh_attachment_normalize_url( $url );

	if ( ! $normalized_url ) {
		return $default_size;
	}

	$attachment_id = mh_get_attachment_id_by_url( $url );

	if ( ! $attachment_id ) {
		return $default_size;
	}

	$metadata = wp_get_attachment_metadata( $attachment_id );

	if ( ! $metadata ) {
		return $default_size;
	}

	$size = $default_size;

	if ( strpos( $url, $metadata['file'] ) === ( strlen( $url ) - strlen( $metadata['file'] ) ) ) {
		$size = array( $metadata['width'], $metadata['height'] );
	} elseif ( preg_match( '/-(\d+)x(\d+)\.(jpg|jpeg|gif|png|svg|webp)$/', $url, $match ) ) {
		$size = array( $match[1], $match[2] );
	}

	return $size;
}
endif;

if ( ! function_exists( 'mh_get_image_srcset_sizes' ) ) :
function mh_get_image_srcset_sizes( $url ) {
	$normalized_url = mh_attachment_normalize_url( $url );

	if ( ! $normalized_url ) {
		return array();
	}

	$attachment_id = mh_get_attachment_id_by_url( $url );
	if ( ! $attachment_id ) {
		return array();
	}

	$image_size = mh_get_attachment_size_by_url( $url );
	if ( ! $image_size ) {
		return array();
	}

	$srcset = wp_get_attachment_image_srcset( $attachment_id, $image_size );
	$sizes  = wp_get_attachment_image_sizes( $attachment_id, $image_size );

	if ( ! $srcset || ! $sizes ) {
		return array();
	}

	$data = array(
		'srcset' => $srcset,
		'sizes'  => $sizes,
	);

	return $data;
}
endif;

if ( ! function_exists( 'mh_attachment_normalize_url' ) ) :

function mh_attachment_normalize_url( $url ) {
	list( $url ) = explode( '?', $url );
  
	$url = str_replace( '%26%23215%3B', 'x', rawurlencode( $url ) );
	$url = rawurldecode( $url );

	if ( 0 !== strpos( $url, 'http' ) ) {
		$wp_upload_dir = wp_upload_dir( null, false );
		$upload_dir    = str_replace( site_url( '/' ), '', $wp_upload_dir['baseurl'] );
		$url_trimmed   = ltrim( $url, '/' );

		if ( 0 === strpos( $url_trimmed, $upload_dir ) || 0 === strpos( $url_trimmed, 'wp-content' ) ) {
			$url = site_url( $url_trimmed );
		} else {
			$url = $wp_upload_dir['baseurl'] . '/' . $url_trimmed;
		}
	}

	if ( ! filter_var( $url, FILTER_VALIDATE_URL ) || ! preg_match( '/^(.+)\.(jpg|jpeg|gif|png|svg|webp)$/', $url ) ) {
		return false;
	}

	return esc_url( $url );
}
endif;

if ( ! function_exists( 'mh_uploads_dir_url' ) ) :
function mh_uploads_dir_url( $url ) {
	$upload_dir = wp_upload_dir( null, false );

	return mh_starts_with( $url, $upload_dir['baseurl'] );
}
endif;

if ( ! function_exists( 'mh_get_src_from_img_tag' ) ) :
function mh_get_src_from_img_tag( $image ) {
	if ( preg_match( '/^<img.+src=[\'"](?P<src>.+?)[\'"].*>/', $image, $match ) ) {
		if ( isset( $match['src'] ) ) {
			return $match['src'];
		}
	}

	if ( class_exists( 'DOMDocument' ) && class_exists( 'DOMXPath' ) ) {
		$doc = new DOMDocument();
		$doc->loadHTML( $image );

		$xpath = new DOMXPath( $doc );
		return $xpath->evaluate( 'string(//img/@src)' );
	}

	return false;
}
endif;

if ( ! function_exists( 'mh_get_image_attachment_class' ) ):
function mh_get_image_attachment_class( $attrs, $source_key, $attachment_id = 0 ) {
		$attachment_class = '';
   if (!mh_use_responsive_images ()){
     if ( ! empty( $source_key ) ) {
			$attachment_src = mh_array_get( $attrs, $source_key, '' );
			$attachment_id  = mh_get_attachment_id_by_url( $attachment_src );
		}
  
		if ( $attachment_id > 0 ) {
			$attachment_class = "wp-image-{$attachment_id}";
		}
   }
  
		

		return $attachment_class;
	}
endif;

if ( ! function_exists( 'mh_get_image_width_height' ) ):
function mh_get_image_width_height($src, $size = 'width'){
    $image   = mh_get_attachment_size_by_url( $src );
    $output = '';
    if ('width' === $size){
        $output  = ( ! empty( $image ) && is_numeric( $image[0] ) ) ? $image[0] : '400'; // default logo width
    }
    if ('height' === $size){			
        $output = ( ! empty( $image ) && is_numeric( $image[1] ) ) ? $image[1] : '78'; // default logo height
    }
    return $output;
}
endif;